@@ -47,7 +47,7 @@ our @EXPORT_OK = ( @{ $EXPORT_TAGS{'all'} } );
 our @EXPORT = qw(
 );
 
-our $VERSION = '0.90';
+our $VERSION = '0.91';
 
 my $debugf;
 
@@ -67,20 +67,22 @@ $Is{BSD}     = ($^O =~ /^(?:free|net|open)bsd$/ or grep( $^O eq $_, qw(bsdos int
 
 =head1 NAME
 
-ExtUtils::MakeMaker::BigHelper - Perl extension for helping ExtUtils::MakeMaker with big XS projects.
+ExtUtils::MakeMaker::BigHelper - for helping ExtUtils::MakeMaker with big XS
+projects.
 
 =head1 SYNOPSIS
 
   use ExtUtils::MakeMaker::BigHelper qw(:find);
 
-    This exports find_files and find_directories, which might be useful with WriteMakefile
-    from ExtUtils::MakeMaker.
+    This exports find_files and find_directories, which might be useful with
+    WriteMakefile from ExtUtils::MakeMaker.
 
   use ExtUtils::MakeMaker::BigHelper qw(:MY);
 
-    Use this after a "package MY;" statement.  It will export methods that are used by
-    ExtUtils::MakeMaker and available for customization.  These customized methods will
-    alter the behaviour of ExtUtils::MakeMaker as documented below.
+    Use this after a "package MY;" statement.  It will export methods that
+    are used by ExtUtils::MakeMaker and available for customization.  These
+    customized methods will alter the behaviour of ExtUtils::MakeMaker as
+    documented below.
 
 =head1 DESCRIPTION
 
@@ -94,20 +96,22 @@ directories are also allowed, hopefully located next to the .pm files they test.
 See the man page for perlxs for more information about perl XS.
 
 With ExtUtils::MakeMaker there can only be one .xs file, which limits the size
-of the project.  Well, there is a way to have more, but you're left with one master
-.xs file responsible for bootstrapping all the other ones.  The way to have two
-.xs files is documented, but not easy, and the way to have more than two is, well, unnatural.
+of the project.  Well, there is a way to have more, but you're left with one
+master .xs file responsible for bootstrapping all the other ones.  The way
+to have two .xs files is documented, but not easy, and the way to have more
+than two is, well, unnatural.
 
 With ExtUtils::MakeMaker you're allowed to provide customizations for various
 MakeMaker methods in the package MY.  This gives you the ability to make a
 total overhaul of ExtUtils::MakeMaker.
 
-This package uses the customization facility built in to ExtUtils::MakeMaker to allow
-multiple .xs files, multiple t files, and multiple subproject (Makefile.PL and Build.PL)
-directories in the lib hierarchy rather than at the top level of the project.  This allows
-you to more conveniently use perlxs in your project, without having to package up multiple
-Makefile.PL projects embedded in your directories.  It also allows you to convert all those
-.pm files with Inline code into real perlxs without using dynamite.
+This package uses the customization facility built in to ExtUtils::MakeMaker
+to allow multiple .xs files, multiple t files, and multiple subproject
+(Makefile.PL and Build.PL) directories in the lib hierarchy rather than at
+the top level of the project.  This allows you to more conveniently use
+perlxs in your project, without having to package up multiple Makefile.PL
+projects embedded in your directories.  It also allows you to convert all
+those .pm files with Inline code into real perlxs without using dynamite.
 
 The methods here are meant to be exported into the package MY, to provide
 customizations of the ExtUtils::MakeMaker methods.
@@ -155,8 +159,14 @@ The Makefile.PL would look like this:
 
   use ExtUtils::MakeMaker::BigHelper qw(:MY);
 
-That should do it.  It uses File::Find to descend the hierarchy and find all the .xs files
-and t directories.
+That should do it.  It uses File::Find to descend the hierarchy and find all
+the .xs files and t directories.
+
+=head2 B<$found = find_files($regex, @directories)>
+
+This function is exported by the :find tag, and looks up all files matching
+the regex, under the listed directories.  It returns them in a hashref where
+the keys are the files found and the value is 1.
 
 =cut
 
@@ -169,6 +179,14 @@ sub find_files {
   return \%found;
 }
 
+=head2 B<$found = find_directories($regex, @directories)>
+
+This function is exported by the :find tag, and looks up all directories
+matching the regex, under the listed directories.  It returns them in a
+hashref where the keys are the directories found and the value is 1.
+
+=cut
+
 sub find_directories {
   my ($pat, @dirs) = @_;
 
@@ -187,6 +205,20 @@ sub make_hashref_of_found_files {
   return \%found;
 }
 
+=head2 B<$obj->init_dirscan>
+
+This function is exported by the :MY tag.  It extends the MM init_dirscan.
+
+The MM init_dirscan only looks for .xs files in the top level directory.
+This extension descends into lib to find all .xs files, and sets them into
+the $self XS setting.
+
+Note that the XS setting is supposed to be a hashref of .xs files,
+where the value is the corresponding .c file.  However, with these
+overrides, the value need only be 1 if the .c file is as expected.
+
+=cut
+
 sub init_dirscan {
   my ($self) = @_;
 
@@ -200,6 +232,23 @@ sub init_dirscan {
   return $result;
 }
 
+=head2 B<$obj->test>
+
+This function is exported by the :MY tag.  It extends the MM test.
+
+The MM test only looks for t directories at the top level directory.  This
+extension descends into lib to find all t directories, and sets them into
+the $self TESTS setting.
+
+The $self TESTS setting is supposed to be a blank separated list
+of t files (with shell wildcards) to run.
+
+Note that the XS setting is supposed to be a hashref of .xs files,
+where the value is the corresponding .c file.  However, with these
+overrides, the value need only be 1 if the .c file is as expected.
+
+=cut
+
 sub test {
   my ($self, %attribs) = @_;
 
@@ -251,6 +300,17 @@ sub test {
 #    $self->MM::init_PM;
 #}
 
+=head2 B<$obj->init_PM>
+
+This function is exported by the :MY tag.  It extends/replaces the MM
+init_PM.
+
+This method only puts .pm files into the $self PM setting.
+
+The default init_PM puts all files into the $self PM setting.
+
+=cut
+
 sub init_PM {
   my ($self) = @_;
 
@@ -342,37 +402,49 @@ sub init_PM {
   }, @{$self->{PMLIBDIRS}});
 }
 
+=head2 B<$obj->init_XS>
+
+This function is exported by the :MY tag.  It extends the MM init_XS.
+
+This sets up macros INST_STATIC, INST_DYNAMIC, INST_BOOT, and uses
+INST_ARCHLIB.
+
+note: init_dirscan will set XS and DIR to defaults.  However, init_dirscan
+has limitations, mostly that it cannot handle large projects.
+
+input and output:
+
+  XS          as documented in ExtUtils::MakeMaker, this is a hashref.
+              the key is the relative path of the xs file.
+              the value can be the target c file as documented.
+              EXTENSION: the value can be simply 1 if the c file can be
+                         simply generated from the xs file.
+              EXTENSION: the value can be the package name XXX::YYY::ZZZ for
+                         the xs code.
+              EXTENSION: the value can be an array of c dependencies
+              EXTENSION: the XS hash is reworked by this code.
+
+  DIR         as documented, an arrayref of subdirectories containing
+              Makefile.PL
+              EXTENSION: may be an arrayref of pathnames for Makefile.PLs
+                         and Build.PLs init_dirscan only looks one deep for
+                         Makefile.PLs, and does not look for Build.PLs.
+                         With this extension, you could do a recursive file
+                         find for all Makefile.PLs and Build.PLs no matter
+                         how deep.
+              
+  MYEXTLIB
+
+output
+
+  MY_XS_DEPENDENCIES
+  MY_SUBDIRS
+  MY_EXTENSION_LIBS
+  MY_XS_TARGETS
+
+=cut
+
 sub init_xs {
-  # sets up macros INST_STATIC, INST_DYNAMIC, INST_BOOT
-  # uses INST_ARCHLIB
-  #
-  # note: init_dirscan will set XS and DIR to defaults.  However, init_dirscan
-  #       has limitations, mostly that it cannot handle large projects.
-  #
-  # input and output:
-  #   XS          as documented in ExtUtils::MakeMaker, this is a hashref.
-  #               the key is the relative path of the xs file.
-  #               the value can be the target c file as documented.
-  #               EXTENSION: the value can be simply 1 if the c file can be
-  #                          simply generated from the xs file.
-  #               EXTENSION: the value can be the package name XXX::YYY::ZZZ for the
-  #                          xs code.
-  #               EXTENSION: the value can be an array of c dependencies
-  #               EXTENSION: the XS hash is reworked by this code.
-  #
-  #   DIR         as documented, an arrayref of subdirectories containing Makefile.PL
-  #               EXTENSION: may be an arrayref of pathnames for Makefile.PLs and Build.PLs
-  #                          init_dirscan only looks one deep for Makefile.PLs, not Build.PLs.
-  #                          with this extension, you could do a recursive file find for all
-  #                          Makefile.PLs and Build.PLs no matter how deep.
-  #               
-  #   MYEXTLIB
-  # output
-  #   MY_XS_DEPENDENCIES
-  #   MY_SUBDIRS
-  #   MY_EXTENSION_LIBS
-  #   MY_XS_TARGETS
-  #
   my $self = shift;
 
   print STDERR "debug: ", Data::Dumper::Dumper($self), "\n" if $debugf;
@@ -489,6 +561,15 @@ sub init_xs {
   print STDERR "debug: my init_xs, after INST_DYNAMIC=$self->{INST_DYNAMIC}\n" if $debugf;
 }
 
+=head2 B<$obj->clean_subdirs>
+
+This function is exported by the :MY tag.  It replaces the MM clean_subdirs.
+
+This handles cleaning up subdirectories with their own Makefile, such
+as Makefile.PL, Build.PL, and extension libraries.
+
+=cut
+
 sub clean_subdirs {
   my $self = shift;
 
@@ -499,13 +580,23 @@ clean_subdirs :
 ';
   foreach my $subdir (@{ $self->{MY_SUBDIRS} } ) {
     $make .= '
-cd '.$subdir.' && $(MAKE) clean
+	cd '.$subdir.' && $(MAKE) clean
 ';
   }
 
   return $make;
 }
 
+=head2 B<$obj->clean>
+
+This function is exported by the :MY tag.  It extends the MM clean.
+
+This handles cleaning up all the debris left by building .xs files,
+without having to have the .c files named explicitly in the $self
+XS setting.
+
+=cut
+
 sub clean {
   my ($self, %attribs) = @_;
 
@@ -527,6 +618,15 @@ sub clean {
   return $make;
 }
 
+=head2 B<$obj->postamble>
+
+This function is exported by the :MY tag.  It extends the MM postamble.
+
+This handles setting up the compilation of the .xs files with a VERSION
+for each.  The bootstrap code checks the VERSION, and it has to match.
+
+=cut
+
 sub postamble {
   my $self = shift;
 
@@ -538,35 +638,35 @@ sub postamble {
     $make .= '
 
 '.$extlib.':
-BUILDDIR=`pwd`/; \
-cd '.$subdir.' && \
-$(MAKE) AR="$(FULL_AR)" ARFLAGS="$(AR_STATIC_ARGS)" RANLIB="$(RANLIB)" CC="$(CCCMD)" CPPFLAGS="-I$(PERL_INC) $(PASTHRU_DEFINE) $(DEFINE)" CFLAGS="$(CCCDLFLAGS)" LD="$(LD)" LDFLAGS="$(LDDLFLAGS)" all
+	BUILDDIR=`pwd`/; \
+	cd '.$subdir.' && \
+	$(MAKE) AR="$(FULL_AR)" ARFLAGS="$(AR_STATIC_ARGS)" RANLIB="$(RANLIB)" CC="$(CCCMD)" CPPFLAGS="-I$(PERL_INC) $(PASTHRU_DEFINE) $(DEFINE)" CFLAGS="$(CCCDLFLAGS)" LD="$(LD)" LDFLAGS="$(LDDLFLAGS)" all
 
 subdirs-test ::
--BUILDDIR=`pwd`/; \
-cd '.$subdir.' && \
-$(MAKE) AR="$(FULL_AR)" ARFLAGS="$(AR_STATIC_ARGS)" RANLIB="$(RANLIB)" CC="$(CCCMD)" CPPFLAGS="-I$(PERL_INC) $(PASTHRU_DEFINE) $(DEFINE)" CFLAGS="$(CCCDLFLAGS)" LD="$(LD)" LDFLAGS="$(LDDLFLAGS)" test
+	-BUILDDIR=`pwd`/; \
+	cd '.$subdir.' && \
+	$(MAKE) AR="$(FULL_AR)" ARFLAGS="$(AR_STATIC_ARGS)" RANLIB="$(RANLIB)" CC="$(CCCMD)" CPPFLAGS="-I$(PERL_INC) $(PASTHRU_DEFINE) $(DEFINE)" CFLAGS="$(CCCDLFLAGS)" LD="$(LD)" LDFLAGS="$(LDDLFLAGS)" test
 
 pure_perl_install ::
--BUILDDIR=`pwd`/; \
-cd '.$subdir.' && \
-$(MAKE) AR="$(FULL_AR)" ARFLAGS="$(AR_STATIC_ARGS)" RANLIB="$(RANLIB)" CC="$(CCCMD)" CPPFLAGS="-I$(PERL_INC) $(PASTHRU_DEFINE) $(DEFINE)" CFLAGS="$(CCCDLFLAGS)" LD="$(LD)" LDFLAGS="$(LDDLFLAGS)" \
-INSTALLPRIVLIB=$(DESTINSTALLPRIVLIB) INSTALLARCHLIB=$(DESTINSTALLARCHLIB) INSTALLBIN=$(DESTINSTALLBIN) INSTALLSCRIPT=$(DESTINSTALLSCRIPT) INSTALLMAN1DIR=$(DESTINSTALLMAN1DIR) INSTALLMAN3DIR=$(DESTINSTALLMAN3DIR) \
-install
+	-BUILDDIR=`pwd`/; \
+	cd '.$subdir.' && \
+	$(MAKE) AR="$(FULL_AR)" ARFLAGS="$(AR_STATIC_ARGS)" RANLIB="$(RANLIB)" CC="$(CCCMD)" CPPFLAGS="-I$(PERL_INC) $(PASTHRU_DEFINE) $(DEFINE)" CFLAGS="$(CCCDLFLAGS)" LD="$(LD)" LDFLAGS="$(LDDLFLAGS)" \
+	INSTALLPRIVLIB=$(DESTINSTALLPRIVLIB) INSTALLARCHLIB=$(DESTINSTALLARCHLIB) INSTALLBIN=$(DESTINSTALLBIN) INSTALLSCRIPT=$(DESTINSTALLSCRIPT) INSTALLMAN1DIR=$(DESTINSTALLMAN1DIR) INSTALLMAN3DIR=$(DESTINSTALLMAN3DIR) \
+	install
 
 pure_site_install ::
--BUILDDIR=`pwd`/; \
-cd '.$subdir.' && \
-$(MAKE) AR="$(FULL_AR)" ARFLAGS="$(AR_STATIC_ARGS)" RANLIB="$(RANLIB)" CC="$(CCCMD)" CPPFLAGS="-I$(PERL_INC) $(PASTHRU_DEFINE) $(DEFINE)" CFLAGS="$(CCCDLFLAGS)" LD="$(LD)" LDFLAGS="$(LDDLFLAGS)" \
-INSTALLPRIVLIB=$(DESTINSTALLSITELIB) INSTALLARCHLIB=$(DESTINSTALLSITEARCHLIB) INSTALLBIN=$(DESTINSTALLSITEBIN) INSTALLSCRIPT=$(DESTINSTALLSITESCRIPT) INSTALLMAN1DIR=$(DESTINSTALLSITEMAN1DIR) INSTALLMAN3DIR=$(DESTINSTALLSITEMAN3DIR) \
-install
+	-BUILDDIR=`pwd`/; \
+	cd '.$subdir.' && \
+	$(MAKE) AR="$(FULL_AR)" ARFLAGS="$(AR_STATIC_ARGS)" RANLIB="$(RANLIB)" CC="$(CCCMD)" CPPFLAGS="-I$(PERL_INC) $(PASTHRU_DEFINE) $(DEFINE)" CFLAGS="$(CCCDLFLAGS)" LD="$(LD)" LDFLAGS="$(LDDLFLAGS)" \
+	INSTALLPRIVLIB=$(DESTINSTALLSITELIB) INSTALLARCHLIB=$(DESTINSTALLSITEARCHLIB) INSTALLBIN=$(DESTINSTALLSITEBIN) INSTALLSCRIPT=$(DESTINSTALLSITESCRIPT) INSTALLMAN1DIR=$(DESTINSTALLSITEMAN1DIR) INSTALLMAN3DIR=$(DESTINSTALLSITEMAN3DIR) \
+	install
 
 pure_vendor_install ::
--BUILDDIR=`pwd`/; \
-cd '.$subdir.' && \
-$(MAKE) AR="$(FULL_AR)" ARFLAGS="$(AR_STATIC_ARGS)" RANLIB="$(RANLIB)" CC="$(CCCMD)" CPPFLAGS="-I$(PERL_INC) $(PASTHRU_DEFINE) $(DEFINE)" CFLAGS="$(CCCDLFLAGS)" LD="$(LD)" LDFLAGS="$(LDDLFLAGS)" \
-INSTALLPRIVLIB=$(DESTINSTALLVENDORLIB) INSTALLARCHLIB=$(DESTINSTALLVENDORARCHLIB) INSTALLBIN=$(DESTINSTALLVENDORBIN) INSTALLSCRIPT=$(DESTINSTALLVENDORSCRIPT) INSTALLMAN1DIR=$(DESTINSTALLVENDORMAN1DIR) INSTALLMAN3DIR=$(DESTINSTALLVENDORMAN3DIR) \
-install
+	-BUILDDIR=`pwd`/; \
+	cd '.$subdir.' && \
+	$(MAKE) AR="$(FULL_AR)" ARFLAGS="$(AR_STATIC_ARGS)" RANLIB="$(RANLIB)" CC="$(CCCMD)" CPPFLAGS="-I$(PERL_INC) $(PASTHRU_DEFINE) $(DEFINE)" CFLAGS="$(CCCDLFLAGS)" LD="$(LD)" LDFLAGS="$(LDDLFLAGS)" \
+	INSTALLPRIVLIB=$(DESTINSTALLVENDORLIB) INSTALLARCHLIB=$(DESTINSTALLVENDORARCHLIB) INSTALLBIN=$(DESTINSTALLVENDORBIN) INSTALLSCRIPT=$(DESTINSTALLVENDORSCRIPT) INSTALLMAN1DIR=$(DESTINSTALLVENDORMAN1DIR) INSTALLMAN3DIR=$(DESTINSTALLVENDORMAN3DIR) \
+	install
 ';
 
   }
@@ -602,8 +702,8 @@ install
     my $tmp_str = '
 
 '.$object.' : '.$xs.'
-$(XSUBPPRUN) $(XSPROTOARG) $(XSUBPPARGS) $*.xs > $*.xsc && $(MV) $*.xsc $*.c
-'.$cccmd.' $(CCCDLFLAGS) "-I$(PERL_INC)" $(PASTHRU_DEFINE) $(DEFINE) -o $@ $*.c
+	$(XSUBPPRUN) $(XSPROTOARG) $(XSUBPPARGS) $*.xs > $*.xsc && $(MV) $*.xsc $*.c
+	'.$cccmd.' $(CCCDLFLAGS) "-I$(PERL_INC)" $(PASTHRU_DEFINE) $(DEFINE) -o $@ $*.c
 ';
     push @compile, $tmp_str;
   }
@@ -622,6 +722,14 @@ MAKE_FRAG
   return $make;
 }
 
+=head2 B<$obj->dynamic_bs>
+
+This function is exported by the :MY tag.  It extends the MM dynamic_bs.
+
+This handles the .bs files for multiple non-chained .xs files.
+
+=cut
+
 sub dynamic_bs {
   my($self, %attribs) = @_;
 
@@ -655,15 +763,15 @@ BOOTSTRAP = $(BASEEXT).bs
 
     my $tmp_str = '
 '.$bootstrap.' : $(FIRST_MAKEFILE) $(BOOTDEP) '.$exists.'
-$(NOECHO) $(ECHO) "Running Mkbootstrap for '.$xs.' ($(BSLOADLIBS))"
-$(NOECHO) $(PERLRUN) "-MExtUtils::Mkbootstrap" -e "Mkbootstrap(\''.$xs_basename.'\',\'$(BSLOADLIBS)\');"
-$(NOECHO) $(TOUCH) %s
-$(CHMOD) $(PERM_RW) %s
+	$(NOECHO) $(ECHO) "Running Mkbootstrap for '.$xs.' ($(BSLOADLIBS))"
+	$(NOECHO) $(PERLRUN) "-MExtUtils::Mkbootstrap" -e "Mkbootstrap(\''.$xs_basename.'\',\'$(BSLOADLIBS)\');"
+	$(NOECHO) $(TOUCH) %s
+	$(CHMOD) $(PERM_RW) %s
 
 '.$install_target.' : '.$bootstrap.' '.$exists.'
-$(NOECHO) $(RM_RF) %s
-- $(CP) '.$bootstrap.' %s
-$(CHMOD) $(PERM_RW) %s
+	$(NOECHO) $(RM_RF) %s
+	- $(CP) '.$bootstrap.' %s
+	$(CHMOD) $(PERM_RW) %s
 ';
 
     $str .= sprintf $tmp_str, ($make_target) x 5;
@@ -672,6 +780,14 @@ $(CHMOD) $(PERM_RW) %s
   return $str;
 }
 
+=head2 B<$obj->dynamic_lib>
+
+This function is exported by the :MY tag.  It extends the MM dynamic_lib.
+
+This handles building multiple .xs files into .so.
+
+=cut
+
 sub dynamic_lib {
   my($self, %attribs) = @_;
 
@@ -689,7 +805,7 @@ sub dynamic_lib {
   my($armaybe) = $attribs{ARMAYBE} || $self->{ARMAYBE} || ":";
   my($ldfrom) = '$(LDFROM)';
   $armaybe = 'ar' if ($Is{OSF} and $armaybe eq ':');
-  my $ld_opt = $Is{OS2} ? '$(OPTIMIZE) ' : '';	# Useful on other systems too?
+  my $ld_opt = $Is{OS2} ? '$(OPTIMIZE) ' : '';  # Useful on other systems too?
   my $ld_fix = $Is{OS2} ? '|| ( $(RM_F) $@ && sh -c false )' : '';
   push(@m,'
 # This section creates the dynamically loadable $(INST_DYNAMIC)
@@ -734,10 +850,10 @@ INST_DYNAMIC_FIX = '.$ld_fix.'
     my $libs = '$(LDLOADLIBS)';
 
     if (($Is{NetBSD} || $Is{Interix}) && $Config{'useshrplib'} eq 'true') {
-# Use nothing on static perl platforms, and to the flags needed
-# to link against the shared libperl library on shared perl
-# platforms.  We peek at lddlflags to see if we need -Wl,-R
-# or -R to add paths to the run-time library search path.
+      # Use nothing on static perl platforms, and to the flags needed
+      # to link against the shared libperl library on shared perl
+      # platforms.  We peek at lddlflags to see if we need -Wl,-R
+      # or -R to add paths to the run-time library search path.
       if ($Config{'lddlflags'} =~ /-Wl,-R/) {
           $libs .= ' -L$(PERL_INC) -Wl,-R$(INSTALLARCHLIB)/CORE -Wl,-R$(PERL_ARCHLIB)/CORE -lperl';
       } elsif ($Config{'lddlflags'} =~ /-R/) {
@@ -758,13 +874,13 @@ INST_DYNAMIC_FIX = '.$ld_fix.'
     }
 
     push @m, sprintf <<'MAKE', $ld_run_path_shell, $ldrun, '$<', join(' ', @extlibs), $libs;
-%s$(LD) %s $(LDDLFLAGS) %s $(OTHERLDFLAGS) -o $@ %s	\
-  $(PERL_ARCHIVE) %s $(PERL_ARCHIVE_AFTER) $(EXPORT_LIST)	\
-  $(INST_DYNAMIC_FIX)
+	%s$(LD) %s $(LDDLFLAGS) %s $(OTHERLDFLAGS) -o $@ %s	\
+	  $(PERL_ARCHIVE) %s $(PERL_ARCHIVE_AFTER) $(EXPORT_LIST)	\
+	  $(INST_DYNAMIC_FIX)
 MAKE
 
     push @m, <<'MAKE';
-$(CHMOD) $(PERM_RWX) $@
+	$(CHMOD) $(PERM_RWX) $@
 MAKE
   }
 
@@ -772,6 +888,7 @@ MAKE
 }
 
 1;
+
 __END__
 
 =head2 EXPORT
